from pyspark.sql import SparkSession
from synapse.ml.lightgbm import LightGBMClassifier
from synapse.ml.train import ComputeModelStatistics

num_cores = 2
chunk_size = 10_000

spark = (
    SparkSession
    .builder
    .master(f"local[{num_cores}]")
    .config("spark.driver.memory", "12g")
    .config("spark.executor.memory", "12g")
    .config("spark.jars.packages", "com.microsoft.azure:synapseml_2.12:0.9.5")
    .config("spark.jars.repositories", "https://mmlspark.azureedge.net/maven")
    .getOrCreate()
)


df = (
    spark.read.format("csv")
    .option("header", True)
    .option("inferSchema", True)
    .load("/opt/spark_data/company_bancruptacy_prediction.csv")
    .repartition(num_cores)
    .cache()
)
# print dataset size
print("records read: " + str(df.count()))
# print("Schema: ")
# df.printSchema()

train, test = df.randomSplit([0.85, 0.15], seed=1)

from pyspark.ml.feature import VectorAssembler
feature_cols = df.columns[1:]
featurizer = VectorAssembler(
    inputCols=feature_cols,
    outputCol='features'
)
train_data = featurizer.transform(train)['Bankrupt?', 'features']
test_data = featurizer.transform(test)['Bankrupt?', 'features']

model = LightGBMClassifier(
    objective="binary",
    featuresCol="features",
    labelCol="Bankrupt?",
    useSingleDatasetMode=True,
    numThreads=max(1, num_cores - 1),
    chunkSize=chunk_size,
    isProvideTrainingMetric=True,
    verbosity=10,
    isUnbalance=True
)

model = model.fit(train_data)

predictions = model.transform(test_data)


metrics = ComputeModelStatistics(
    evaluationMetric="classification",
    labelCol='Bankrupt?',
    scoredLabelsCol='prediction'
).transform(predictions)

print(f"METRICS: {metrics.collect()}")

spark.stop()